package mi.hdm.mealPlan;

import mi.hdm.recipes.Recipe;
import mi.hdm.recipes.ValidObjectsPool;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.time.LocalDate;
import java.util.HashMap;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class MealPlanTest {
    private MealPlan underTest;
    private final static Recipe recipeOne = ValidObjectsPool.getValidRecipeOne();
    private final static Recipe recipeTwo = ValidObjectsPool.getValidRecipeTwo();

    @BeforeEach
    public void setup() {
        underTest = new MealPlan(new HashMap<>());
    }

    @Test
    public void canAddToMealPlan() {
        //given
        LocalDate date = LocalDate.now();
        Map<LocalDate, Recipe> expected = Map.of(date, recipeOne);

        //when
        underTest.addRecipeToMealPlan(recipeOne, date);

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }

    @Test
    public void shouldNotAddWhenDateNull() {
        assertThrows(NullPointerException.class, () -> underTest.addRecipeToMealPlan(recipeOne, null));
    }

    @Test
    public void shouldNotAddWhenOutOfBounds() {
        //given
        LocalDate future = LocalDate.of(3000, 1, 1);
        LocalDate past = LocalDate.of(2000, 1, 1);

        //expect throw
        assertThrows(UnsupportedOperationException.class, () -> underTest.addRecipeToMealPlan(recipeOne, future));
        assertThrows(UnsupportedOperationException.class, () -> underTest.addRecipeToMealPlan(recipeOne, past));
    }

    @Test
    public void shouldRemoveOldRecipes() {
        //given
        LocalDate date = LocalDate.of(2022, 1, 1);
        Map<LocalDate, Recipe> expected = Map.of();
        HashMap<LocalDate, Recipe> entries = new HashMap<>();
        entries.put(date, recipeOne);
        underTest = new MealPlan(entries);

        //when
        underTest.removePastRecipes();

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }

    @Test
    public void shouldRemoveOldRecipesButKeepUpToDate() {
        //given
        LocalDate old = LocalDate.of(2022, 1, 1);
        LocalDate now = LocalDate.now();
        HashMap<LocalDate, Recipe> entries = new HashMap<>();
        entries.put(old, recipeOne);
        entries.put(now, recipeTwo);
        underTest = new MealPlan(entries);

        Map<LocalDate, Recipe> expected = Map.of(now, recipeTwo);

        //when
        underTest.removePastRecipes();

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }

    @Test
    public void shouldClearDayIfPresent() {
        //given
        Map<LocalDate, Recipe> expected = Map.of();
        LocalDate date = LocalDate.now();
        underTest.addRecipeToMealPlan(recipeOne, date);

        //when
        underTest.clear(date);

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }

    @Test
    public void shouldThrowOnClearInvalidDate() {
        //given
        LocalDate date = LocalDate.now();

        //expect
        assertThrows(NullPointerException.class, () -> underTest.clear(null));
        assertThrows(NullPointerException.class, () -> underTest.clear(date));
    }
}
